package nl.overheid.stelsel.digimelding.interceptors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.ws.WebServiceMessage;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.context.MessageContext;
import org.xml.sax.InputSource;

import javax.xml.transform.*;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.*;

/**
 * An interceptor that helps to retrieve the raw request and response when performing a SOAP request.
 */
@Component
public class MessageInterceptor implements ClientInterceptor {

    private static final Logger LOGGER = LoggerFactory.getLogger(MessageInterceptor.class);
    private static final int INDENT_NUMBER = 4;

    private PropertyChangeListener requestListener;

    private String request;
    private String response;

    /**
     * Retrieves the (raw) sent request and tells listeners that the request is avalailable (for example to show in GUI).
     *
     * @param messageContext - The MessageContext for the performed request.
     *
     * @throws WebServiceClientException
     */
    @Override
    public boolean handleRequest(MessageContext messageContext) throws WebServiceClientException {
        String oldValue = this.request;
        this.request = toPrettyXML(getXMLMessageFromWebServiceMessage(messageContext.getRequest()));
        fireRequestSent("request", oldValue, this.request);
        return true;
    }

    /**
     * Retrieves the (raw) received response.
     *
     * @param messageContext - The MessageContext for the performed request.
     *
     * @throws WebServiceClientException
     */
    @Override
    public boolean handleResponse(MessageContext messageContext) throws WebServiceClientException {
        this.response = toPrettyXML(getXMLMessageFromWebServiceMessage(messageContext.getResponse()));
        return true;
    }

    /**
     * Retrieves the (raw) received response in case of a fault.
     *
     * @param messageContext - The MessageContext for the performed request.
     *
     * @throws WebServiceClientException
     */
    @Override
    public boolean handleFault(MessageContext messageContext) throws WebServiceClientException {
        this.response = toPrettyXML(getXMLMessageFromWebServiceMessage(messageContext.getResponse()));
        return false;
    }

    @Override
    public void afterCompletion(MessageContext messageContext, Exception e) throws WebServiceClientException {
        //Do nothing because already handled.
    }

    /**
     * Resets the request and response.
     */
    public void clearRequestAndResponse(){
        this.request = null;
        this.response = null;
    }

    /**
     * Sets the listener which wants to know when the sent request is known.
     * @param listener - The listener
     */
    public void setRequestListener(PropertyChangeListener listener) {
        this.requestListener = listener;
    }

    /**
     * Retrieves the content of a WebServiceMessage in pretty xml.
     *
     * @param msg - The WebServiceMessage to retrieve the content from
     *
     * @return String
     */
    private String getXMLMessageFromWebServiceMessage(WebServiceMessage msg){
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            msg.writeTo(out);
        } catch (IOException e) {
            LOGGER.error(e.toString(), e);
        }

        return new String(out.toByteArray());
    }

    /**
     * Reads the xml response of the webservice and turns it into pretty xml.
     *
     * @param xml - The string of xml to make pretty
     *
     * @return - String
     */
    private String toPrettyXML(String xml) {
        StringWriter stringWriter = new StringWriter();
        String beforeXMLPart = "";
        String afterXMLPart = "";

        final String START_MESSAGE = "<soap:Envelope";
        final String END_MESSAGE = "</soap:Envelope>";

        boolean isEnvelopeResponse = xml.contains(START_MESSAGE);

        if(isEnvelopeResponse) {
            beforeXMLPart = xml.substring(0, xml.indexOf(START_MESSAGE));
            afterXMLPart = xml.substring(xml.indexOf(END_MESSAGE) + END_MESSAGE.length(), xml.length());
            xml =  xml.substring(xml.indexOf(START_MESSAGE), xml.indexOf(END_MESSAGE) + END_MESSAGE.length());
        }

        transform(xml, stringWriter);

        return beforeXMLPart + stringWriter.toString() + afterXMLPart;
    }

    /**
     * Fires the propertychange event, so the listener will know the request is sent.
     *
     * @param property - The name (string) of the variable/property that is changed.
     * @param oldValue - The old value of the property.
     * @param newValue - The new value of the property.
     */
    private void fireRequestSent(String property, Object oldValue, Object newValue) {
        if(requestListener != null) {
            this.requestListener.propertyChange(new PropertyChangeEvent(this, property, oldValue, newValue));
        }
    }

    private void transform(String xml, StringWriter stringWriter) {
        try {
            InputSource in = new InputSource(new StringReader(xml));
            SAXSource source = new SAXSource(in);

            Transformer transformer = getTransformer();
            transformer.transform(source, new StreamResult(stringWriter));

            return;
        } catch(TransformerException te){
            LOGGER.error(te.toString(), te);
        }
    }

    private Transformer getTransformer() throws TransformerConfigurationException {
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        transformerFactory.setAttribute("indent-number", INDENT_NUMBER);

        Transformer transformer = transformerFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");

        return transformer;
    }

    public String getRequest() {
        return this.request;
    }

    public String getResponse() {

        return this.response;
    }
}
