package nl.overheid.stelsel.digimelding.interceptors;

import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;
import org.springframework.ws.soap.security.wss4j2.Wss4jSecurityInterceptor;
import org.springframework.ws.soap.security.wss4j2.support.CryptoFactoryBean;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * An interceptor that helps set securement actions to be performed when sending a SOAP request.
 */
@Component
public class SecurityRequestInterceptor extends Wss4jSecurityInterceptor {

    private int timestamp;
    private String signatureAlgorithm;
    private String digestAlgorithm;

    private CryptoFactoryBean cryptoFactoryBean;
    private SettingsManager settingsManager;

    private static final Logger LOGGER = getLogger(SecurityRequestInterceptor.class);
    private static final String SECUREMENT_ACTIONS = "Timestamp Signature";
    private static final String SIGNATURE_KEY_IDENTIFIER = "DirectReference";

    @Autowired
    public SecurityRequestInterceptor(@Value("${digimelding.timestamp-request}") int timestamp,
            @Value("${digimelding.signature-algorithm}") String signatureAlgorithm,
            @Value("${digimelding.digest-algorithm}") String digestAlgorithm, SettingsManager settingsManager){
        this.cryptoFactoryBean = new CryptoFactoryBean();
        this.settingsManager = settingsManager;
        this.timestamp = timestamp;
        this.signatureAlgorithm = signatureAlgorithm;
        this.digestAlgorithm = digestAlgorithm;

        setInitialSecurement();
    }

    /**
     * At startup no password are provided so the expected error is not thrown, but only logged instead.
     */
    private void setInitialSecurement(){
        setSecurementActions(SECUREMENT_ACTIONS);

        try {
            setSecurement();
        } catch (InvalidKeystoreSettingsException e) {
            LOGGER.warn(e.getMessage());
        }
    }

    /**
     * Sets the securement actions for the SOAP request.
     * Makes sure to use 'Timestamp', as well as 'Signature'
     *
     * For more info:
     * https://docs.spring.io/spring-ws/site/apidocs/org/springframework/ws/soap/security/wss4j/Wss4jSecurityInterceptor.html
     */
    public void setSecurement() throws InvalidKeystoreSettingsException {
        setCrypto();

        setSecurementSignatureAlgorithm(signatureAlgorithm);
        setSecurementSignatureDigestAlgorithm(digestAlgorithm);
        setSecurementSignatureKeyIdentifier(SIGNATURE_KEY_IDENTIFIER);

        setSecurementTimeToLive(timestamp);

        setSecurementUsername(settingsManager.getClientCertificateAlias());
        setSecurementPassword(settingsManager.getKeyStorePassword());
    }

    /**
     * Makes sure the right keystore is used and there is access to it.
     * Makes sure the right signature is used for the request.
     *
     * @throws InvalidKeystoreSettingsException - thrown when an invalid password or keystore is provided.
     */
    private void setCrypto() throws InvalidKeystoreSettingsException {
        try {
            this.cryptoFactoryBean.setKeyStorePassword(settingsManager.getKeyStorePassword());
            this.cryptoFactoryBean.setKeyStoreLocation(new ClassPathResource(settingsManager.getKeyStorePath()));

            System.setProperty("javax.net.ssl.keyStore", settingsManager.getKeyStorePath());
            System.setProperty("javax.net.ssl.keyStorePassword", settingsManager.getKeyStorePassword());

            this.cryptoFactoryBean.afterPropertiesSet();
            setSecurementSignatureCrypto(this.cryptoFactoryBean.getObject());
        } catch(Exception e){
            throw new InvalidKeystoreSettingsException("The keystore/password combination that is set is not correct!", e);
        }
    }
}
