package nl.overheid.stelsel.digimelding.interceptors;

import nl.overheid.stelsel.digimelding.exceptions.InvalidCryptoSettingsException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;
import org.springframework.ws.soap.security.wss4j2.Wss4jSecurityInterceptor;
import org.springframework.ws.soap.security.wss4j2.support.CryptoFactoryBean;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * An interceptor that helps set validation actions to be performed when receiving a SOAP response.
 */
@Component
public class SecurityResponseInterceptor extends Wss4jSecurityInterceptor {

    private CryptoFactoryBean cryptoFactoryBean;
    private SettingsManager settingsManager;

    private static final Logger LOGGER = getLogger(SecurityResponseInterceptor.class);
    private static final String VALIDATION_ACTIONS = "Timestamp Signature";

    @Autowired
    public SecurityResponseInterceptor(SettingsManager settingsManager) {
        this.cryptoFactoryBean = new CryptoFactoryBean();
        this.settingsManager = settingsManager;

        setInitialValidation();
    }

    /**
     * At startup no password are provided so the expected error is not thrown, but only logged instead.
     */
    private void setInitialValidation(){
        setValidationActions(VALIDATION_ACTIONS);
        try {
            setCrypto();
        } catch (InvalidCryptoSettingsException | InvalidKeystoreSettingsException e) {
            LOGGER.warn(e.getMessage());
        }
    }

    /**
     * Sets the validation actions for the SOAP request.
     * Makes sure to use 'Timestamp', as well as 'Signature'
     *
     * For more info:
     * https://docs.spring.io/spring-ws/site/apidocs/org/springframework/ws/soap/security/wss4j/Wss4jSecurityInterceptor.html
     */
    public void setValidation() throws Exception {
        setCrypto();
    }

    /**
     * Makes sure the right trusttore is used and there is access to it.
     * Makes sure the signature of the response is validated in the correct way.
     *
     * @throws Exception - thrown in case of invalid settings or truststore.
     */
    private void setCrypto() throws InvalidCryptoSettingsException, InvalidKeystoreSettingsException {
        try {
            this.cryptoFactoryBean.setKeyStorePassword(this.settingsManager.getTrustStorePassword());
            this.cryptoFactoryBean.setKeyStoreLocation(new ClassPathResource(this.settingsManager.getTrustStorePath()));

            System.setProperty("javax.net.ssl.trustStore", this.settingsManager.getTrustStorePath());

            cryptoFactoryBean.afterPropertiesSet();
            setValidationSignatureCrypto(cryptoFactoryBean.getObject());

        } catch(Exception e){
            ensureValidationSignature();

            throw new InvalidKeystoreSettingsException("The truststore/password combination that is set is not correct!", e);
        }
    }

    private void ensureValidationSignature() throws InvalidCryptoSettingsException {
        CryptoFactoryBean backupDecryptoFactoryBean = new CryptoFactoryBean();
        try {
            backupDecryptoFactoryBean.afterPropertiesSet();
            setValidationSignatureCrypto(backupDecryptoFactoryBean.getObject());
        } catch(Exception e){
            throw new InvalidCryptoSettingsException("Validation actions could not be set!", e);
        }
    }
}
