package nl.overheid.stelsel.digimelding.settings;

import nl.overheid.stelsel.digimelding.exceptions.EmptyKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import org.apache.commons.lang.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.*;

/**
 * Class for setting and retrieving settings used in the application.
 * Settings can be read from configuration files or set directly.
 */
public class SettingsManager {

    private String oin;
    private String trustStorePath;
    private String keyStorePath;
    private String trustStorePassword;
    private String keyStorePassword;
    private String annotatieEndpoint;
    private String catalogusEndpoint;
    private String clientCertificateAlias;

    private X509Certificate clientCertificate;
    private List<X509Certificate> trustedCertificates;

    /**
     * Sets the client certificate used for requests by retrieving it from the keystore.
     *
     * @param keyStorePath - The location of the keystore.
     * @param keyStorePassword - The password of the keystore.
     *
     * @throws EmptyKeystoreException - thrown when there is no certificate in the keystore
     * @throws InvalidKeystoreException - thrown when the keystore is not valid or broken.
     * @throws InvalidKeystoreSettingsException - thrown when invalid settings are provided (like invalid password).
     */
    public void setClientCertificate(String keyStorePath, String keyStorePassword) throws EmptyKeystoreException, InvalidKeystoreException, InvalidKeystoreSettingsException, IOException {
        Map.Entry<String, X509Certificate> certificate =
                getCertificatesKeyStore(keyStorePath, keyStorePassword).entrySet().iterator().next();
        this.clientCertificate = certificate.getValue();
        this.clientCertificateAlias = certificate.getKey();
        this.keyStorePath = keyStorePath;
        this.oin = StringUtils.substringBetween(clientCertificate.getSubjectDN().toString(), "SERIALNUMBER=", ",");
        this.keyStorePassword = keyStorePassword;
    }

    /**
     * Sets the trusted certificates by setting the truststore.
     *
     * @param trustStorePath - The location of the truststore.
     * @param trustStorePassword - The password of the truststore.
     *
     * @throws EmptyKeystoreException - thrown when there is no certificate in the truststore
     * @throws InvalidKeystoreException - thrown when the truststore is not valid or broken.
     * @throws InvalidKeystoreSettingsException - thrown when invalid settings are provided (like invalid password).
     */
    public void setTrustedCertificates(String trustStorePath, String trustStorePassword) throws EmptyKeystoreException, InvalidKeystoreException, InvalidKeystoreSettingsException, IOException {
        this.trustStorePath = trustStorePath;
        trustedCertificates =
                new ArrayList<>(getCertificatesKeyStore(trustStorePath, trustStorePassword).values());
        this.trustStorePassword = trustStorePassword;
    }

    public String getClientCertificateAlias(){
        return this.clientCertificateAlias;
    }

    public X509Certificate getClientCertificate(){
        return this.clientCertificate;
    }

    public List<X509Certificate> getTrustedCertificates(){
        return this.trustedCertificates;
    }

    public String getOin() {
        return oin;
    }

    public String getTrustStorePath() {
        return trustStorePath;
    }

    public String getKeyStorePath() {
        return keyStorePath;
    }

    public String getAnnotatieEndpoint() {
        return annotatieEndpoint;
    }

    public void setAnnotatieEndpoint(String annotatieEndpoint) {
        this.annotatieEndpoint = annotatieEndpoint;
    }

    public String getCatalogusEndpoint() {
        return catalogusEndpoint;
    }

    public void setCatalogusEndpoint(String catalogusEndpoint) {
        this.catalogusEndpoint = catalogusEndpoint;
    }

    public String getTrustStorePassword() {
        return trustStorePassword;
    }

    public String getKeyStorePassword() {
        return keyStorePassword;
    }

    /**
     * Retrieves certificates from a keystore.
     *
     * @param keyStorePath - The location of the keystore.
     * @param keyStorePassword - The password of the keystore.
     *
     * @return Map with the certificates.
     * The alias of the certificate in the keystore is used as the key, the certificate itself is the value.
     *
     * @throws EmptyKeystoreException - thrown when there is no certificate in the truststore
     * @throws InvalidKeystoreException - thrown when the truststore is not valid or broken.
     * @throws InvalidKeystoreSettingsException - thrown when invalid settings are provided (like invalid password).
     */
    private Map<String, X509Certificate> getCertificatesKeyStore(String keyStorePath, String keyStorePassword) throws EmptyKeystoreException, InvalidKeystoreException, InvalidKeystoreSettingsException, IOException {
        Map<String, X509Certificate> certificates = new HashMap<>();

        FileInputStream is = null;
        try {
            File file = new File(keyStorePath);
            is = new FileInputStream(file);

            KeyStore keystore = KeyStore.getInstance(KeyStore.getDefaultType());
            keystore.load(is, keyStorePassword.toCharArray());

            Enumeration enumeration = keystore.aliases();
            while (enumeration.hasMoreElements()) {
                String alias = (String) enumeration.nextElement();
                certificates.put(alias, (X509Certificate) keystore.getCertificate(alias));
            }
        } catch(IOException ioe) {
            throw new InvalidKeystoreSettingsException(ioe.getMessage(), ioe);
        } catch (Exception e) {
            throw new InvalidKeystoreException(e.getMessage(), e);
        } finally {
            if (null != is) {
                is.close();
            }
        }
        if(certificates.size() == 0){
            throw new EmptyKeystoreException("No certificates found");
        }
        return certificates;
    }
}
