package nl.overheid.stelsel.digimelding.interceptors;

import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import org.apache.wss4j.common.crypto.Crypto;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.File;
import java.io.FileReader;
import java.util.Properties;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

public class SecurityRequestInterceptorIT {

    private SecurityRequestInterceptor securityRequestInterceptor;
    private SecurityResponseInterceptor securityResponseInterceptorMock;

    private SettingsManager settingsManagerMock;

    private static Properties p;

    @BeforeClass
    public static void setUpClass() throws Exception{
        p = new Properties();
        System.out.println(new File("src/test/resources/test.properties").getAbsolutePath());
        p.load(new FileReader(new File("src/test/resources/test.properties")));
    }

    @Before
    public void setUp() {
        settingsManagerMock = Mockito.mock(SettingsManager.class);
        securityResponseInterceptorMock = Mockito.mock(SecurityResponseInterceptor.class);
        when(settingsManagerMock.getClientCertificateAlias()).thenReturn("test");

        int timestamp = Integer.parseInt(p.getProperty("digimelding.timestamp-request"));
        String signatureAlgorithm = p.getProperty("digimelding.signature-algorithm");
        String digestAlgorithm = p.getProperty("digimelding.digest-algorithm");

        this.securityRequestInterceptor = new SecurityRequestInterceptor(timestamp, signatureAlgorithm, digestAlgorithm,
                settingsManagerMock);
    }

    @Test
    public void testSetSecumentWithEmptyKeystore() throws Exception {
        when(settingsManagerMock.getKeyStorePassword()).thenReturn("password");
        when(settingsManagerMock.getKeyStorePath()).thenReturn("keystore.p12");

        SecurityRequestInterceptor spy = Mockito.spy(this.securityRequestInterceptor);

        spy.setSecurement();
        verify(spy, times(1)).setSecurementSignatureCrypto(any(Crypto.class));
        verify(spy, times(1)).setSecurementSignatureAlgorithm("http://www.w3.org/2000/09/xmldsig#rsa-sha1");
        verify(spy, times(1)).setSecurementSignatureDigestAlgorithm("http://www.w3.org/2000/09/xmldsig#sha1");
        verify(spy, times(1)).setSecurementSignatureKeyIdentifier("DirectReference");

        verify(spy, times(1)).setSecurementTimeToLive(30000);

        verify(spy, times(1)).setSecurementUsername("test");
        verify(spy, times(1)).setSecurementPassword("password");
    }

    @Test
    public void testSetSecumentWithWrongPasswordForKeystoreThrowInvalidKeystoreSettingsException() throws Exception {
        when(settingsManagerMock.getKeyStorePassword()).thenReturn("wrong-password");
        when(settingsManagerMock.getKeyStorePath()).thenReturn("keystore.p12");

        SecurityRequestInterceptor spy = Mockito.spy(this.securityRequestInterceptor);

        try {
            spy.setSecurement();
            fail("Should have thrown exception");
        } catch(Exception e){
            assertEquals(e.getClass(), InvalidKeystoreSettingsException.class);
        }

        verify(spy, times(0)).setSecurementSignatureCrypto(any(Crypto.class));
        verify(spy, times(0)).setSecurementSignatureAlgorithm(any(String.class));
    }

    @Test
    public void testSetSecumentWithNonExistingKeystoreThrowInvalidKeystoreSettingsException() throws Exception {
        when(settingsManagerMock.getKeyStorePassword()).thenReturn("password");
        when(settingsManagerMock.getKeyStorePath()).thenReturn("non-existing-keystore.p12");

        SecurityRequestInterceptor spy = Mockito.spy(this.securityRequestInterceptor);

        try {
            spy.setSecurement();
            fail("Should have thrown exception");
        } catch(Exception e){
            assertEquals(e.getClass(), InvalidKeystoreSettingsException.class);
        }

        verify(spy, times(0)).setSecurementSignatureCrypto(any(Crypto.class));
        verify(spy, times(0)).setSecurementSignatureAlgorithm(any(String.class));
    }

}
