package nl.overheid.stelsel.digimelding.settings;

import nl.overheid.stelsel.digimelding.exceptions.EmptyKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import org.junit.Before;
import org.junit.Test;

import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

public class SettingsManagerTest {
    private SettingsManager settingsManager;

    @Before
    public void setUp(){
        this.settingsManager = new SettingsManager();
    }

    @Test
    public void setClientCertificate() throws IOException, InvalidKeystoreSettingsException, InvalidKeystoreException, EmptyKeystoreException {
        settingsManager.setClientCertificate("src/test/resources/keystore.p12", "password");

        assertEquals("CN=test, OU=test, O=test, L=test, ST=test, C=te",
                settingsManager.getClientCertificate().getSubjectDN().toString());
        assertEquals("test", settingsManager.getClientCertificateAlias());
        assertEquals("password", settingsManager.getKeyStorePassword());
        assertEquals("src/test/resources/keystore.p12", settingsManager.getKeyStorePath());
    }

    @Test
    public void setTrustedCertificates() throws IOException, InvalidKeystoreSettingsException, InvalidKeystoreException, EmptyKeystoreException {
        settingsManager.setTrustedCertificates("src/test/resources/truststore.p12", "password");

        List<X509Certificate> certificates = settingsManager.getTrustedCertificates();
        assertEquals(1, certificates.size());

        X509Certificate serverCertificate = certificates.get(0);

        assertEquals("CN=test, OU=test, O=test, L=test, ST=test, C=te",
                serverCertificate.getSubjectDN().toString());
        assertEquals("src/test/resources/truststore.p12", settingsManager.getTrustStorePath());
        assertEquals("password", settingsManager.getTrustStorePassword());
    }

    @Test
    public void setClientCertificateWithWrongKeystorePasswordThrowsInvalidKeystoreSettingsException() {
        try {
            settingsManager.setClientCertificate("src/test/resources/empty-keystore.p12", "wrong-password");
            fail("Should have thrown an exception");
        } catch(InvalidKeystoreSettingsException e) {
            assertNull(settingsManager.getKeyStorePassword());
            assertNull(settingsManager.getKeyStorePath());
        } catch(Exception e){
            fail("Threw the wrong exception");
        }
    }

    @Test
    public void setClientCertificateFromEmptyKeystoreThrowsEmptyKeystoreException() {
        try {
            settingsManager.setClientCertificate("src/test/resources/empty-keystore.p12", "password");
            fail("Should have thrown an exception");
        } catch(EmptyKeystoreException e) {
            assertNull(settingsManager.getKeyStorePassword());
            assertNull(settingsManager.getKeyStorePath());
        } catch(Exception e){
            fail("Threw the wrong exception");
        }
    }

    @Test
    public void testAnnotationEndpoint(){
        settingsManager.setAnnotatieEndpoint("annotation-endpoint");
        String endpoint = settingsManager.getAnnotatieEndpoint();

        assertEquals("annotation-endpoint", endpoint);
    }

    @Test
    public void testCatalogusEndpoint(){
        settingsManager.setCatalogusEndpoint("catalogus-endpoint");
        String endpoint = settingsManager.getCatalogusEndpoint();

        assertEquals("catalogus-endpoint", endpoint);
    }
}
