package nl.overheid.stelsel.digimelding.controllers;

import de.felixroske.jfxsupport.FXMLController;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.stage.FileChooser;
import nl.overheid.stelsel.digimelding.App;
import nl.overheid.stelsel.digimelding.clients.AnnotatieClient;
import nl.overheid.stelsel.digimelding.exceptions.EmptyKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreException;
import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import nl.overheid.stelsel.digimelding.interceptors.SecurityRequestInterceptor;
import nl.overheid.stelsel.digimelding.interceptors.SecurityResponseInterceptor;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import nl.overheid.stelsel.digimelding.views.EchoView;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;

@FXMLController
public class LoginController extends MainController {

    @Value("${spring.profiles.active}")
    private String profile;

    private PropertiesConfiguration conf;

    @FXML
    private TextField locationKeyStoreText;

    @FXML
    private PasswordField passwordKeyStoreText;

    @FXML
    private TextField locationTrustStoreText;

    @FXML
    private PasswordField passwordTrustStoreText;

    private SettingsManager settingsManager;
    private SecurityRequestInterceptor securityRequestInterceptor;
    private SecurityResponseInterceptor securityResponseInterceptor;

    private static final String DEFAULT_DIRECTORY = "/";

    @Autowired
    public LoginController(SettingsManager settingsManager, SecurityRequestInterceptor securityRequestInterceptor,
                           SecurityResponseInterceptor securityResponseInterceptor){
        this.settingsManager = settingsManager;
        this.securityRequestInterceptor = securityRequestInterceptor;
        this.securityResponseInterceptor = securityResponseInterceptor;
    }

    @FXML
    protected void initialize() throws ConfigurationException {
        conf = new PropertiesConfiguration("application-" + profile + ".properties");

        locationKeyStoreText.setText(conf.getProperty("digimelding.ws.key-store").toString());
        locationTrustStoreText.setText(conf.getProperty("digimelding.ws.trust-store").toString());
    }

    @FXML
    private void pickKeyStoreLocation(final Event event) {
        FileChooser fileChooser = getFileChooser("Keystores", new String[]{"*.jks", "*.p12"}, DEFAULT_DIRECTORY);
        File file = fileChooser.showOpenDialog(locationKeyStoreText.getScene().getWindow());

        if (file != null) {
            String location = file.getPath();

            if(!location.equals(locationKeyStoreText.getText())) {
                locationKeyStoreText.setText(location);
                passwordKeyStoreText.clear();
            }
        }
    }

    @FXML
    private void pickTrustStoreLocation(final Event event) {
        FileChooser fileChooser = getFileChooser("Keystores", new String[]{"*.jks", "*.p12"}, DEFAULT_DIRECTORY);
        File file = fileChooser.showOpenDialog(locationTrustStoreText.getScene().getWindow());

        if (file != null) {
            String location = file.getPath();

            if(!location.equals(locationTrustStoreText.getText())) {
                locationTrustStoreText.setText(location);
                passwordTrustStoreText.clear();
            }
        }
    }

    @FXML
    private void saveSettings(final Event event) throws Exception {
        conf.setProperty("digimelding.ws.key-store", locationKeyStoreText.getText());
        conf.setProperty("digimelding.ws.trust-store", locationTrustStoreText.getText());

        try {
            conf.save();

            settingsManager.setClientCertificate(locationKeyStoreText.getText(), passwordKeyStoreText.getText());
            settingsManager.setTrustedCertificates(locationTrustStoreText.getText(), passwordTrustStoreText.getText());

            securityRequestInterceptor.setSecurement();
            securityResponseInterceptor.setValidation();

            App.showView(EchoView.class);
        } catch (ConfigurationException e) {
            alertHelper.generalErrorAlert(e);
        } catch (InvalidKeystoreSettingsException | EmptyKeystoreException | InvalidKeystoreException e) {
            alertHelper.invalidSecuritySettingsAlert(e, "Onjuiste instellingen!");
        }
    }

    private FileChooser getFileChooser(String description, String[] extensions, String directory){
        final FileChooser fileChooser = new FileChooser();

        fileChooser.getExtensionFilters().addAll(
                new FileChooser.ExtensionFilter(description, extensions));
        fileChooser.setInitialDirectory(new File(directory));

        return fileChooser;
    }
}
