package nl.overheid.stelsel.digimelding.controllers;

import javafx.concurrent.Task;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.paint.Color;
import nl.overheid.stelsel.digimelding.actions.ActionResult;
import nl.overheid.stelsel.digimelding.commands.ICommand;
import nl.overheid.stelsel.digimelding.exceptions.InvalidSettingsException;
import org.slf4j.Logger;
import org.springframework.oxm.UnmarshallingFailureException;
import org.springframework.ws.client.WebServiceIOException;
import org.springframework.ws.soap.client.SoapFaultClientException;

import java.net.ConnectException;
import java.net.URISyntaxException;

import static org.slf4j.LoggerFactory.getLogger;

public class RequestController extends MainController {

    private static final Logger LOGGER = getLogger(RequestController.class);

    protected Label status;
    protected TextArea request;
    protected TextArea response;
    protected Button send;

    protected void clearRequestAndResponse(TextArea request, TextArea response){
        request.clear();
        response.clear();
    }

    protected void executeRequest(ICommand command){
        setBusyStatus();

        Task<Object> task = new Task<Object>() {
            @Override
            public Object call() throws URISyntaxException, InvalidSettingsException {
                if(!command.canExecute()){
                    throw new IllegalArgumentException();
                }
                command.execute();

                ActionResult result = command.getAction().getResult();
                if (result == null){ throw new InvalidSettingsException();}
                return result;
            }
        };

        task.setOnSucceeded(e -> {
            setPositiveRequestStatus();
            return;
        });
        task.setOnFailed(e -> {
            //Because the Binary Security Token in the response is not added inline, validation of the certificate/signature
            //is not working correctly.
            //This check will ignore this error, resulting in the certificate sent with the response not being checked with the trust store!
            if(task.getException() instanceof UnmarshallingFailureException){
                ignoreFailingSignatureCheckResponseBecauseOfNoInlineBinarySecurityToken();
                return;
            }
            setNegativeRequestStatus();
            handleException((Exception) task.getException());
        });

        new Thread(task).start();
    }

    private void ignoreFailingSignatureCheckResponseBecauseOfNoInlineBinarySecurityToken(){
        setPositiveRequestStatus();
    }

    private void setRequestAndResponse() {
        this.response.setText(messageInterceptor.getResponse());
        messageInterceptor.clearRequestAndResponse();
    }

    public void setBusyStatus(){
        this.status.setText("Bezig");
        this.status.setTextFill(Color.BLACK);
        this.send.setDisable(true);
        clearRequestAndResponse(this.request, this.response);
    }

    public void setPositiveRequestStatus(){
        this.status.setText("Succes!");
        this.status.setTextFill(Color.GREEN);
        this.send.setDisable(false);
        setRequestAndResponse();
    }

    public void setNegativeRequestStatus(){
        this.status.setText("Error!");
        this.status.setTextFill(Color.RED);
        this.send.setDisable(false);
        setRequestAndResponse();
    }


    protected void handleException(Exception e){
        if(e instanceof InvalidSettingsException){
            alertHelper.invalidSecuritySettingsAlert(e, "Niet verzonden!");
        } else if(e instanceof IllegalArgumentException) {
            alertHelper.missingInformationAlert("Alle velden dienen te worden ingevuld!");
        } else if(e instanceof SoapFaultClientException) {
            alertHelper.invalidInformationAlert(e);
        } else if (e instanceof WebServiceIOException) {
            this.response.setText("");
            if(e.getCause() instanceof ConnectException){
                alertHelper.noConnectionPossibleAlert(e);
            } else {
                alertHelper.generalErrorAlert("Bekijk zorgvuldig de foutmelding! \n\n" +
                        "Controleer onder meer of uw certificaten en/of keystores geldig zijn!", e);
            }
        } else {
            alertHelper.generalErrorAlert(e);
        }

        LOGGER.error("There was an error sending the request", e);
        LOGGER.info("The sent request is:\n" + this.request.getText());
        LOGGER.info("The received response is:\n" + this.response.getText());
    }
}
