﻿using System;
using System.Text;
using System.ServiceModel.Channels;
using System.IO;
using System.Xml;
using Core.Digimelding.Koppeling.Inspectors;
using System.Configuration;

namespace Core.Digimelding.Koppeling.Bindings
{
    /// <summary>
    /// CustomMtomBindingElement which makes sure the right encoders are used for the requests and responses.
    /// </summary>
    public class CustomMtomBindingElement : MessageEncodingBindingElement
    {
        private MessageVersion _messageVersion = MessageVersion.Soap11WSAddressing10;
        private IMessageInspector inspector;

        public CustomMtomBindingElement(IMessageInspector inspector)
        {
            this.inspector = inspector;
        }

        public override MessageEncoderFactory CreateMessageEncoderFactory()
        {
            return new CustomMtomEncoderFactory(this._messageVersion, this.inspector);
        }
        public override MessageVersion MessageVersion
        {
            get
            {
                return this._messageVersion;
            }
            set
            {
                this._messageVersion = value;
            }
        }
        public override BindingElement Clone()
        {
            return this;
        }
        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }
        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            context.BindingParameters.Add(this);
            return context.BuildInnerChannelListener<TChannel>();
        }
        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            return context.CanBuildInnerChannelFactory<TChannel>();
        }
        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            return context.CanBuildInnerChannelListener<TChannel>();
        }
    }
    class CustomMtomEncoderFactory : MessageEncoderFactory
    {
        MessageVersion messageVersion;
        CustomMtomEncoder encoder;

        public CustomMtomEncoderFactory(MessageVersion messageVersion, IMessageInspector inspector)
        {
            this.messageVersion = messageVersion;
            this.encoder = new CustomMtomEncoder(messageVersion, inspector);
        }
        public override MessageEncoder Encoder
        {
            get
            {
                return this.encoder;
            }
        }
        public override MessageVersion MessageVersion
        {
            get
            {
                return this.encoder.MessageVersion;
            }
        }
    }
    class CustomMtomEncoder : MessageEncoder
    {
        MessageEncoder _textEncoder;
        MessageEncoder _mtomEncoder;
        private IMessageInspector messageInspector;
        private static readonly int MAX_BUFFER_SIZE = 30000;

        public CustomMtomEncoder(MessageVersion messageVersion, IMessageInspector inspector)
        {
            int bufferSize = Convert.ToInt32(ConfigurationManager.AppSettings["max-buffer-size"]);
            _textEncoder = new TextMessageEncodingBindingElement(messageVersion, Encoding.UTF8).CreateMessageEncoderFactory().Encoder;
            _mtomEncoder = new MtomMessageEncodingBindingElement(messageVersion, Encoding.UTF8) {
                MaxBufferSize = bufferSize > 0 ? bufferSize : MAX_BUFFER_SIZE
            }.CreateMessageEncoderFactory().Encoder;
            this.messageInspector = inspector;
        }
        public override string ContentType
        {
            get
            {
                return _textEncoder.ContentType;
            }
        }
        public override string MediaType
        {
            get
            {
                return _textEncoder.MediaType;
            }
        }
        public override MessageVersion MessageVersion
        {
            get
            {
                return _textEncoder.MessageVersion;
            }
        }
        public override bool IsContentTypeSupported(string contentType)
        {
            return this._mtomEncoder.IsContentTypeSupported(contentType);
        }
        public override T GetProperty<T>()
        {
            T result = this._textEncoder.GetProperty<T>();
            if (result == null)
            {
                result = this._mtomEncoder.GetProperty<T>();
            }
            if (result == null)
            {
                result = base.GetProperty<T>();
            }
            return result;
        }
        public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string contentType)
        {
            Message result = this._mtomEncoder.ReadMessage(buffer, bufferManager, contentType);

            //Result is the response from the Webservice, it is set on the messageInspector.
            this.messageInspector.receivedMessage = result.ToString();
            return result;
        }
        public override ArraySegment<byte> WriteMessage(Message message, int maxMessageSize, BufferManager bufferManager, int messageOffset)
        {
            ArraySegment<byte> buffer = this._textEncoder.WriteMessage(message, maxMessageSize, bufferManager, messageOffset);

            //Set the request to the Webservice on the messageInspector.
            this.messageInspector.setRequest(getFullMessage(message, buffer, bufferManager));
            return buffer;
        }
        private static bool IsMtomMessage(string contentType)
        {
            return contentType.IndexOf("type=\"application/xop+xml\"", StringComparison.OrdinalIgnoreCase) >= 0;
        }
        private bool ShouldWriteMtom(Message message)
        {
            return false;
        }
        private XmlDictionaryWriter CreateMtomWriter(Stream stream, Message message)
        {
            return XmlDictionaryWriter.CreateMtomWriter(stream, Encoding.UTF8, int.MaxValue, "text/xml", null, "", true, true);
        }

        public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string contentType)
        {
            Message result = this._mtomEncoder.ReadMessage(stream, maxSizeOfHeaders, contentType);
            return result;
        }

        public override void WriteMessage(Message message, Stream stream)
        {
            _textEncoder.WriteMessage(message, stream);
        }
        /// <summary>
        /// Retrieves the sent message including SOAP (security) headers from 'buffer'. 
        /// The body of the message is not included and is therefore added by retrieving it from 'message', which is the request before adding SOAP (security) headers.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="buffer">A ArraySegment<T> of type Byte that provides the buffer from which the message is deserialized.</param>
        /// <param name="bufferManager">The BufferManager that manages the buffer from which the message is deserialized.</param>
        /// <returns></returns>
        private String getFullMessage(Message message, ArraySegment<byte> buffer, BufferManager bufferManager)
        {
            byte[] msgContents = new byte[buffer.Count];
            Array.Copy(buffer.Array, buffer.Offset, msgContents, 0, msgContents.Length);
            bufferManager.ReturnBuffer(buffer.Array);

            MemoryStream stream = new MemoryStream(msgContents);
            Message m = this._textEncoder.ReadMessage(stream, int.MaxValue);

            String request = m.ToString();

            //The body of the message is not a part of 'request' (instead it shows '... stream ...') because it is a stream which can only be read once and is therefore ignored.
            //The body is filled here with the original body set in 'message'
            String body = "\n    <" + message.ToString().Split(new string[] { "<s:Body" }, StringSplitOptions.None)[1]
                .Split(new char[] { '<' }, 2)[1]
                .Split(new string[] { "</s:Body>" }, StringSplitOptions.None)[0];

            return request.Replace("... stream ...", body);
        }
    }
}