﻿using System;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text.RegularExpressions;

namespace Core.Digimelding.Koppeling
{
    /// <summary>
    /// A manager which enables the application to retrieve configurable settings for the application.
    /// </summary>
    public class SettingsManager : ISettingsManager
    {
        private X509Certificate2 clientCertificate;
        private X509Certificate2 serverCertificate;

        public string Oin { get; set; }
        public string ServerCertificatePath { get; set; }
        public string ClientCertificatePath { get; set; }
        public string ClientCertificatePassword { get; set; }
        public string AnnotatieEndpoint { get; set; }
        public string AnnotatieDns { get; set; }
        public string CatalogusEndpoint { get; set; }
        public string CatalogusDns { get; set; }

        public X509Certificate2 getClientCertificate()
        {
            return this.clientCertificate;
        }

        /// <summary>
        /// Creates a certificate object based on the provided path and password for the certificate.
        /// Also sets the OIN of off the certificate subject.
        /// 
        /// Throws an error when an invalid certificate or password is provided.
        /// </summary>
        /// <param name="clientCertificatePath">Path to the certificate/key pair (.p12)</param>
        /// <param name="clientCertificatePassword">Required password</param>
        public void setClientCertificate(string clientCertificatePath, string clientCertificatePassword)
        {
            this.ClientCertificatePath = clientCertificatePath;

            this.clientCertificate = new X509Certificate2(clientCertificatePath, clientCertificatePassword,
                    X509KeyStorageFlags.Exportable | X509KeyStorageFlags.PersistKeySet);
            
            this.ClientCertificatePassword = clientCertificatePassword;

            setOin();
        }

        public X509Certificate2 getServerCertificate()
        {
            return this.serverCertificate;
        }

        /// <summary>
        /// Creates a certificate object based on the provided path and password for the certificate.
        /// </summary>
        /// <param name="serverCertificatePath">Path to the certificate/key pair</param>
        public void setServerCertificate(string serverCertificatePath)
        {
            this.ServerCertificatePath = serverCertificatePath;
            this.serverCertificate = new X509Certificate2(serverCertificatePath);
        }

        /// <summary>
        /// Retrieves the OIN from the client certificate subject.
        /// </summary>
        private void setOin()
        {
            this.Oin = Regex.Match(this.clientCertificate.Subject, @"SERIALNUMBER=(.+?),").Groups[1].Value;

            if (!isValidOin())
            {
                throw new InvalidOperationException("The certificate does not contain a valid OIN!");
            }
        }

        /// <summary>
        /// Verifies if the OIN consists of 20 numbers.
        /// </summary>
        /// <returns>True or False</returns>
        private bool isValidOin()
        {
            return this.Oin.Length == 20 && this.Oin.All(char.IsDigit);
        }
    }
}
