﻿using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Inspectors;
using System;
using System.Drawing;
using System.IdentityModel.Tokens;
using System.ServiceModel;
using System.ServiceModel.Security;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using Voorbeeldkoppeling.Dialogs;

namespace Voorbeeldkoppeling.Controls
{
    public abstract class RequestControl : UserControl
    {
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public IMessageInspector MessageInspector { get; set; }

        protected Label status;
        protected TextBox request;
        protected TextBox response;
        protected Button send;

        public RequestControl() {}

        protected async void executeRequest(ICommand command)
        {
            this.MessageInspector.RequestSent += new EventHandler(setRequest);

            if (command.canExecute())
            {
                setScreenToRequestBusy();

                try
                {
                    await Task.Run(() =>
                    {
                        command.Execute();
                    });

                    setScreenToRequestSuccess();
                }
                catch (EndpointNotFoundException we)
                {
                    handleException(we, true, false);
                }
                catch (MessageSecurityException mse)
                {
                    handleException(mse, true, true);
                }
                catch (Exception e)
                {
                    handleException(e, true, true);
                }
            }
            else
            {
                missingInfoError("Vul alle velden in!");
            }
        }

        private void setScreenToRequestSuccess()
        {
            this.status.Content = "Succes!";
            this.status.Foreground = System.Windows.Media.Brushes.Green;

            this.send.IsEnabled = true;
            this.response.Text = this.MessageInspector.receivedMessage;
            this.request.Text = this.MessageInspector.sentMessage;
        }

        private void setScreenToRequestBusy()
        {
            this.status.Content = "Bezig";
            this.status.Foreground = System.Windows.Media.Brushes.Black;
            this.response.Text = "";
            this.request.Text = "";
            this.send.IsEnabled = false;
        }

        private void handleException(Exception e, bool showRequest = false, bool showResponse = false)
        {
            log.Error(e);

            if(showRequest) log.Info("The sent request: \n" + this.MessageInspector.sentMessage);
            if(showResponse) log.Info("The received response: \n" + this.MessageInspector.receivedMessage);

            this.status.Content = "Error!";
            this.status.Foreground = System.Windows.Media.Brushes.Red;

            this.send.IsEnabled = true;
            this.response.Text = this.MessageInspector.receivedMessage;
            this.request.Text = this.MessageInspector.sentMessage;

            if (e.GetType() == typeof(EndpointNotFoundException)) {
                new StacktraceDialog("Geen verbinding!",
                    "Controleer uw firewallinstellingen en of uw toegang heeft tot Digimelding vanaf uw netwerk!", e.ToString(), SystemIcons.Error);
            } else if (e.GetType() == typeof(MessageSecurityException)) {
                if (e.InnerException == null) {
                    new StacktraceDialog("Niet verzonden!", "De foutmelding was: \n\n" + e.Message, e.ToString(), SystemIcons.Error);
                } else if (e.InnerException.GetType() == typeof(FaultException)) {
                    new StacktraceDialog("Fout!", "De ontvangen fout is: \n\n" + e.InnerException + "\n\nZie de respons voor het volledige bericht.",
                        e.ToString(), SystemIcons.Information);
                } else {
                    new StacktraceDialog("Er ging iets mis!", "De ontvangen fout is: \n\n" + e.InnerException, e.ToString(), SystemIcons.Information);
                }
            } else if (e.GetType() == typeof(InvalidOperationException)) {
                new StacktraceDialog("Niet verzonden!", "Controleer de instellingen bij 'Instellingen', mogelijke oorzaken: \n\n" +
                        "  - Een verkeerde (locatie naar een) clientcertificaat toegevoegd\n" +
                        "  - Het wachtwoord klopt niet\n" +
                        "  - Een verkeerde (locatie naar een) server certificaat toegevoegd", e.ToString(), SystemIcons.Error);
            } else if (e.GetType() == typeof(SecurityTokenValidationException)) {
                new StacktraceDialog("Er ging iets mis!", "Controleer de instellingen bij 'Instellingen', mogelijke oorzaken: \n\n" +
                        "  - Een verkeerde (locatie naar een) servercertificaat toegevoegd", e.ToString(), SystemIcons.Information);
            } else {
                new StacktraceDialog("Er ging iets mis", e.Message, e.ToString(), SystemIcons.Error);
            }
        }
        protected void missingInfoError(string message)
        {
            this.response.Text = "";
            this.request.Text = "";

            this.status.Content = "Error!";
            this.status.Foreground = System.Windows.Media.Brushes.Red;

            MessageBox.Show(message, "Niet verzonden!", MessageBoxButton.OK, MessageBoxImage.Information);

            log.Info(message);
        }

        protected void setRequest(object sender, EventArgs e)
        {
            Application.Current.Dispatcher.Invoke(setRequest);
        }

        private void setRequest()
        {
            this.request.Text = MessageInspector.sentMessage;
        }
    }
}
